package nl.iobyte.themepark.api.attraction;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.attraction.enums.Status;
import nl.iobyte.themepark.api.attraction.objects.Attraction;
import nl.iobyte.themepark.api.attraction.objects.Region;
import nl.iobyte.themepark.api.event.attraction.AttractionCreateEvent;
import nl.iobyte.themepark.api.event.attraction.AttractionRemoveEvent;
import nl.iobyte.themepark.api.event.region.RegionCreateEvent;
import nl.iobyte.themepark.api.event.region.RegionRemoveEvent;
import nl.iobyte.themepark.api.message.Text;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

public class AttractionService {

    private final LinkedHashMap<String, Region> regions = new LinkedHashMap<>();
    private final Map<String, Attraction> attractions = new HashMap<>();

    /**
     * Add Region
     * @param region Region
     */
    public void addRegion(Region region) {
        if(region.getID() == null || region.getID().isEmpty())
            return;

        if(hasRegion(region.getID()))
            return;

        regions.put(region.getID(), region);
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new RegionCreateEvent(
                region
        ));
    }

    /**
     * Has Region with ID
     * @param id String
     * @return Boolean
     */
    public boolean hasRegion(String id) {
        if(id == null || id.isEmpty())
            return false;

        return regions.containsKey(id);
    }

    /**
     * Get Region by ID
     * @param id String
     * @return Region
     */
    public Region getRegion(String id) {
        if(id == null || id.isEmpty())
            return null;

        return regions.get(id);
    }

    /**
     * Get Map with Regions
     * @return LinkedHashMap<String, Region>
     */
    public LinkedHashMap<String, Region> getRegions() {
        return regions;
    }

    /**
     * Remove Region by ID
     * @param id String
     * @return Region
     */
    public Region removeRegion(String id) {
        Region region = regions.remove(id);
        if(region == null)
            return null;

        for(Attraction attraction : region.getAttractions().values())
            attractions.remove(attraction.getID());

        ThemePark.getInstance().getAPI().getEventDispatcher().call(new RegionRemoveEvent(
                region
        ));

        return region;
    }

    /**
     * Add Attraction
     * @param attraction Attraction
     */
    public void addAttraction(Attraction attraction) {
        if(attraction.getID() == null || attraction.getID().isEmpty())
            return;

        if(hasAttraction(attraction.getID()))
            return;

        if(!hasRegion(attraction.getRegionID()))
            return;

        getRegion(attraction.getRegionID()).addAttraction(attraction);
        attractions.put(attraction.getID(), attraction);
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new AttractionCreateEvent(
                attraction
        ));
    }

    /**
     * Has Attraction with ID
     * @param id String
     * @return Boolean
     */
    public boolean hasAttraction(String id) {
        if(id == null || id.isEmpty())
            return false;

        return attractions.containsKey(id);
    }

    /**
     * Get Attraction by ID
     * @param id String
     * @return Attraction
     */
    public Attraction getAttraction(String id) {
        if(id == null || id.isEmpty())
            return null;

        return attractions.get(id);
    }

    /**
     * Get Attraction by Name
     * @param name String
     * @return Attraction
     */
    public Attraction getAttractionByName(String name) {
        if(name == null || name.isEmpty())
            return null;

        name = Text.strip(name);
        for(Attraction attraction : attractions.values())
            if(Text.strip(attraction.getName()).equals(name))
                return attraction;

        return null;
    }

    public Map<String, Attraction> getAttractionsByStatus(Status status) {
        if(status == null)
            return null;

        Map<String, Attraction> map = new HashMap<>();
        for(Map.Entry<String, Attraction> entrySet : attractions.entrySet())
            if(entrySet.getValue().getStatus() == status)
                map.put(entrySet.getKey(), entrySet.getValue());

        return map;
    }

    /**
     * Get Map with Attractions
     * @return Map<String, Attraction>
     */
    public Map<String, Attraction> getAttractions() {
        return attractions;
    }

    /**
     * Remove Attraction by ID
     * @param id String
     * @return Attraction
     */
    public Attraction removeAttraction(String id) {
        Attraction attraction = attractions.remove(id);
        if(attraction == null)
            return null;

        getRegion(attraction.getRegionID()).removeAttraction(id);
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new AttractionRemoveEvent(
                attraction
        ));

        return attraction;
    }

}
