package nl.iobyte.themepark.api.config.objects;

import nl.iobyte.themepark.logger.ThemeParkLogger;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.plugin.Plugin;
import java.io.File;
import java.util.List;

public class Configuration {

    private final Plugin plugin;
    private File file;
    private FileConfiguration config;
    private final String fileName;

    public Configuration(Plugin plugin, String fileName, boolean update) {
        this.plugin = plugin;
        this.fileName = fileName;
        load(update);
    }

    /**
     * Load config from file
     */
    private void load(boolean update) {
        file = new File(plugin.getDataFolder(), fileName);
        if(!file.getParentFile().exists()) {
            if (!file.getParentFile().mkdirs()) {
                ThemeParkLogger.toConsole("Unable to create parent directories for config file: " + fileName);
                return;
            }
        }

        if(!file.exists()) {
            if(plugin.getResource(fileName) == null) {
                try {
                    if(!file.createNewFile())
                        ThemeParkLogger.toConsole("Unable to create config file: " + fileName);
                } catch(Exception e) {
                    ThemeParkLogger.toConsole("Unable to create config file: " + fileName);
                    return;
                }
            } else {
                plugin.saveResource(fileName, true);
            }
        }

        if(fileName.endsWith(".json")) {
            config = JsonConfiguration.loadConfiguration(file);
        } else {
            config = YamlConfiguration.loadConfiguration(file);
        }

        if(!update)
            return;

        new ConfigurationUpdater(this).update();
    }

    /**
     * Get plugin
     * @return Plugin
     */
    public Plugin getPlugin() {
        return plugin;
    }

    /**
     * Get file name of configuration
     * @return String
     */
    public String getFileName() {
        return fileName;
    }

    /**
     * Save config to file
     */
    public void save() {
        try {
            config.save(file);
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Delete config file
     */
    public void delete() {
        if(!file.delete())
            ThemeParkLogger.toConsole("Unable to remove file: "+fileName);
    }

    /**
     * Check if config contains path
     * @param key Path
     * @return Boolean
     */
    public boolean contains(String key) {
        return config.contains(key);
    }

    /**
     * Set config value
     * @param key Path to value
     * @param value Value
     */
    public void set(String key, Object value) {
        config.set(key, value);
    }

    /**
     * Get object
     * @param key Path to object
     * @return Object
     */
    public Object get(String key) {
        return config.get(key);
    }

    /**
     * Get string
     * @param key Path to string
     * @return String
     */
    public String getString(String key) {
        return config.getString(key);
    }

    /**
     * Get integer
     * @param key Path to integer
     * @return Integer
     */
    public int getInt(String key) {
        return config.getInt(key);
    }

    /**
     * Get boolean
     * @param key Path to boolean
     * @return Boolean
     */
    public boolean getBoolean(String key) {
        return config.getBoolean(key);
    }

    /**
     * Get List of strings
     * @param key Path to list
     * @return List of strings
     */
    public List<String> getStringList(String key) {
        return config.getStringList(key);
    }

    /**
     * Get section of config
     * @param key Path to section
     * @return Section of config
     */
    public ConfigurationSection getSection(String key) {
        return config.getConfigurationSection(key);
    }

}
