package nl.iobyte.themepark.api.load.objects;

import com.cryptomorin.xseries.XMaterial;
import com.google.common.base.Strings;
import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.ThemeParkAPI;
import nl.iobyte.themepark.api.attraction.AttractionService;
import nl.iobyte.themepark.api.attraction.enums.Status;
import nl.iobyte.themepark.api.attraction.objects.Attraction;
import nl.iobyte.themepark.api.attraction.objects.Region;
import nl.iobyte.themepark.api.config.enums.StorageKey;
import nl.iobyte.themepark.api.config.objects.Configuration;
import nl.iobyte.themepark.api.config.ConfigurationManager;
import nl.iobyte.themepark.api.config.enums.StorageLocation;
import nl.iobyte.themepark.api.database.objects.Database;
import nl.iobyte.themepark.api.load.interfaces.IDataLoader;
import nl.iobyte.themepark.utils.LocationUtil;
import org.bukkit.Location;
import org.bukkit.configuration.ConfigurationSection;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

public class StatusDataLoader implements IDataLoader {

    /**
     * Called when loading data
     */
    public void load() {
        ThemeParkAPI api = ThemePark.getInstance().getAPI();
        ConfigurationManager manager = api.getConfigurationManager();

        //Load data
        loadStatus(manager);
        loadRegions(api, manager);

        //Insert into Database
        prepareDatabase(api);
    }

    private void loadStatus(ConfigurationManager manager) {
        if(!manager.contains(StorageLocation.STATUS_SETTINGS, "states"))
            return;

        ConfigurationSection section = manager.getSection(StorageLocation.STATUS_SETTINGS, "states");
        if(section.getKeys(false).isEmpty())
            return;

        Status status;
        String path;
        for(String id : section.getKeys(false)) {
            status = Status.get(id);
            if(status == null)
                continue;
            
            path = "states."+id;
            if(manager.contains(StorageLocation.STATUS_SETTINGS, path+".name"))
                status.setName(manager.getString(StorageLocation.STATUS_SETTINGS, path+".name"));

            if(manager.contains(StorageLocation.STATUS_SETTINGS, path+".color"))
                status.setColor(manager.getString(StorageLocation.STATUS_SETTINGS, path+".color"));

            if(manager.contains(StorageLocation.STATUS_SETTINGS, path+".hex_color"))
                status.setHexColor(manager.getString(StorageLocation.STATUS_SETTINGS, path+".hex_color"));

            if(manager.contains(StorageLocation.STATUS_SETTINGS, path+".teleport"))
                status.setCanTeleport(manager.getBoolean(StorageLocation.STATUS_SETTINGS, path+".teleport"));

            XMaterial material = null;
            if(manager.contains(StorageLocation.STATUS_SETTINGS, path+".material"))
                material = XMaterial.matchXMaterial(manager.getString(StorageLocation.STATUS_SETTINGS, path + ".material")).orElse(XMaterial.RED_CONCRETE);
            
            if(material == null)
                continue;
            
            status.setMaterial(material);
        }
    }

    /**
     * Load Regions from config
     * @param api ThemeParkAPI
     * @param manager ConfigurationManager
     */
    private void loadRegions(ThemeParkAPI api, ConfigurationManager manager) {
        //See if any regions exist
        AttractionService attractionService = api.getAttractionService();
        if(!manager.contains(StorageLocation.REGIONS, "regions"))
            return;

        ConfigurationSection section = manager.getSection(StorageLocation.REGIONS, "regions");
        if(section.getKeys(false).isEmpty())
            return;

        //Load regions
        for(String id : section.getKeys(false)) {
            XMaterial material = XMaterial.matchXMaterial(
                    Optional.ofNullable(
                            manager.getString(StorageLocation.REGIONS, "regions."+id+".material")
                    ).orElse("NAME_TAG")
            ).orElse(XMaterial.NAME_TAG);
            String name = manager.getString(StorageLocation.REGIONS, "regions."+id+".name");

            Region region = new Region(id, name, material.parseMaterial());
            attractionService.addRegion(region);
            loadAttractions(api, region); //try loading Attractions for Region
        }
    }

    /**
     * Load Attractions for specific Region
     * @param api ThemeParkAPI
     * @param region Region
     */
    private void loadAttractions(ThemeParkAPI api, Region region) {
        //See if any attractions exist
        Configuration config = region.getConfiguration();
        if(!config.contains("attractions"))
            return;

        ConfigurationSection section = config.getSection("attractions");
        if(section == null || section.getKeys(false).isEmpty())
            return;

        //Load Attractions
        for(String id : section.getKeys(false)) {
            String name = config.getString("attractions."+id+".name");
            String cover = config.getString("attractions."+id+".cover");
            if(cover == null || cover.isEmpty())
                cover = "https://via.placeholder.com/500x500";

            Status status = Status.get(config.getString("attractions."+id+".status"));
            if(status == null)
                status = Status.CLOSED;

            Location location = LocationUtil.fromString(config.getString("attractions."+id+".location"));

            Attraction attraction = new Attraction(id, region.getID(), name, cover, status, location);
            api.getAttractionService().addAttraction(attraction);
        }
    }

    private void prepareDatabase(ThemeParkAPI api) {
        AttractionService service = api.getAttractionService();
        Database db = api.getDatabaseService().getDatabase("remote");

        //Variables
        int size;
        Map<Integer, Object> parameters = new HashMap<>();

        //Prepare Status Query Parameters
        for(Status status : Status.values()) {
            size = parameters.size();
            parameters.put(size + 1, status.getID());
            parameters.put(size + 2, status.getName());
            parameters.put(size + 3, status.getHexColor());
        }

        //Execute Status Query
        db.executeAsync(
                "INSERT IGNORE INTO states(id, name, color) VALUES "+ Strings.repeat("(?,?,?),", Status.values().length - 1)+"(?,?,?)",
                new HashMap<>(parameters)
        );

        //Prepare Region Query Parameters
        parameters.clear();
        for(Region region : service.getRegions().values()) {
            size = parameters.size();
            parameters.put(size + 1, region.getID());
            parameters.put(size + 2, region.getName());
        }

        if(parameters.isEmpty())
            return;

        //Execute Region Query
        db.executeAsync(
                "INSERT IGNORE INTO regions(id, name) VALUES "+ Strings.repeat("(?,?),", service.getRegions().values().size() - 1)+"(?,?)",
                new HashMap<>(parameters)
        );

        //Prepare Attraction Query Parameters
        parameters.clear();
        for(Attraction attraction : service.getAttractions().values()) {
            size = parameters.size();
            parameters.put(size + 1, attraction.getID());
            parameters.put(size + 2, attraction.getRegionID());
            parameters.put(size + 3, attraction.getName());
            parameters.put(size + 4, attraction.getCover());
            parameters.put(size + 5, attraction.getStatus().getID());
        }

        if(parameters.isEmpty())
            return;

        //Execute Region Query
        db.executeAsync(
                "INSERT IGNORE INTO attractions(id, region_id, name, cover, status_id) VALUES "+ Strings.repeat("(?,?,?,?,?),", service.getAttractions().values().size() - 1)+"(?,?,?,?,?)",
                parameters
        );
    }

}
