package nl.iobyte.themepark.api.menu.objects;

import com.google.common.collect.Iterables;
import nl.iobyte.menuapi.enums.Types;
import nl.iobyte.menuapi.item.ItemBuilder;
import nl.iobyte.menuapi.item.MenuItem;
import nl.iobyte.menuapi.multi.MenuPage;
import nl.iobyte.menuapi.multi.MultiMenu;
import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.ThemeParkAPI;
import nl.iobyte.themepark.api.attraction.enums.Status;
import nl.iobyte.themepark.api.attraction.objects.Attraction;
import nl.iobyte.themepark.api.attraction.objects.Region;
import nl.iobyte.themepark.api.config.enums.StorageKey;
import nl.iobyte.themepark.api.menu.objects.actions.PageAction;
import nl.iobyte.themepark.api.menu.objects.actions.TPAction;
import nl.iobyte.themepark.api.message.MessageKey;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import java.util.*;

public class StatusMenu {

    private MultiMenu menu;
    private final HashMap<String, List<Integer>> region_index = new HashMap<>();
    private final HashMap<String, Integer> attraction_index = new HashMap<>();

    /**
     * Load the menu
     */
    public void load() {
        ThemeParkAPI api = ThemePark.getInstance().getAPI();
        region_index.clear();
        attraction_index.clear();

        //Prepare data
        LinkedHashMap<Integer, Region> regions = new LinkedHashMap<>();
        LinkedHashMap<Integer, List<Attraction>> attractions = new LinkedHashMap<>();
        for(Region region : api.getAttractionService().getRegions().values()) {
            if(region.getAttractions().isEmpty()) {
                regions.put(regions.size(), region);
                attractions.put(attractions.size(), new ArrayList<>());
                continue;
            }


            Iterable<List<Attraction>> iterable = Iterables.partition(region.getAttractions().values(), 8);
            for(List<Attraction> list : iterable) {
                regions.put(regions.size(), region);
                attractions.put(attractions.size(), list);
            }
        }

        //Prepare menu
        loadMenuPages(api, regions.size());

        //Load menu content
        loadMenuContent(regions, attractions);

        //Load next/previous page items
        loadPageItems();

        //Build Menu
        menu.build();
    }

    /**
     * Update Region item
     * @param region Region
     */
    public void updateRegion(Region region) {
        if(region == null)
            return;

        if(!region_index.containsKey(region.getID()))
            return;

        List<Integer> indexes = region_index.get(region.getID());
        if(indexes == null || indexes.isEmpty())
            return;

        for(int slot : indexes) {
            menu.setItem(slot, getRegion(region));
            menu.updateItem(slot);
        }
    }

    /**
     * Update Attraction item
     * @param attraction Attraction
     */
    public void updateAttraction(Attraction attraction) {
        if(attraction == null)
            return;

        int slot = attraction_index.getOrDefault(attraction.getID(), 0);
        if(slot < 1)
            return;

        menu.setItem(slot, getAttraction(attraction));
        menu.updateItem(slot);
    }

    /**
     * Update Attraction item where Status
     * @param status Status
     */
    public void updateStatus(Status status) {
        for(Map.Entry<String, Attraction> entry : ThemePark.getInstance().getAPI().getAttractionService().getAttractionsByStatus(status).entrySet()) {
            int slot = attraction_index.getOrDefault(entry.getKey(), 0);
            if(slot < 1)
                return;

            menu.setItem(slot, getAttraction(entry.getValue()));
            menu.updateItem(slot);
        }
    }

    /**
     * Open menu on first page for Player
     * @param player Player
     */
    public void open(Player player) {
        open(player, 1);
    }

    /**
     * Open menu for Player
     * @param player Player
     * @param page Integer
     */
    public void open(Player player, int page) {
        menu.open(player, page);
    }

    /**
     * Initialize menu and page sizes
     * @param api ThemeParkAPI
     * @param regions Integer
     */
    private void loadMenuPages(ThemeParkAPI api, int regions) {
        ArrayList<Integer> page_sizes = new ArrayList<>();
        if(regions <= 6) {
            if(regions == 0)
                regions = 1;

            page_sizes.add(regions * 9);
        } else {
            int pages = regions / 5;
            for (int i = 0; i < pages; i++)
                page_sizes.add(54);

            pages = regions % 5;
            if (pages > 0)
                page_sizes.add(pages * 9 + 9);
        }

        menu = new MultiMenu(api.getConfigurationManager().getString(StorageKey.MENU_TITLE_STATUS), page_sizes);
    }

    /**
     * Add regions and attractions to menu
     * @param regions LinkedHashMap<Integer, Region>
     * @param attractions LinkedHashMap<Integer, List<Attraction>>
     */
    private void loadMenuContent(LinkedHashMap<Integer, Region> regions, LinkedHashMap<Integer, List<Attraction>> attractions) {
        int region_size = regions.size();
        for(int i = 0; i < region_size; i++) {
            Region region = regions.get(i);
            if(region == null)
                continue;

            int j = 1;
            int start_index = i * 9;
            if(region_size > 6)
                start_index += (i / 5) * 9;

            //Add item index to map
            List<Integer> indexes = region_index.computeIfAbsent(region.getID(), s -> new ArrayList<>());
            indexes.add(start_index);
            region_index.put(region.getID(), indexes);

            //Add item to menu
            menu.setItem(start_index, new MenuItem(getRegion(region), true));

            List<Attraction> list = attractions.get(i);
            if(list == null)
                continue;

            //Add attractions to menu
            for(Attraction attraction : list) {
                MenuItem item = new MenuItem(getAttraction(attraction), true);
                item.addActions(Types.NORMAL.getTypesList(), new TPAction(attraction));

                int slot = start_index + j;
                attraction_index.put(attraction.getID(), slot);
                menu.setItem(slot, item);
                j++;
            }
        }
    }

    /**
     * Load next/previous page items
     */
    private void loadPageItems() {
        int page_size = menu.getPageSizes().size();
        for (int i = 0; i < page_size; i++) {
            MenuPage page = menu.getPage(i + 1);
            int size = page.getSize();
            if (i > 0)
                page.setItem(size - 6, getPrevious(i));

            if (i < (page_size - 1))
                page.setItem(size - 4, getNext(i));

            if(page_size > 1) {
                ItemBuilder builder = new ItemBuilder(Material.MINECART);
                builder.setName(MessageKey.MENU_PAGE.getMessage().replaceAll("%PAGE%", Integer.toString(i + 1)));
                page.setItem(size - 5, new MenuItem(builder.getItem(), true));
            }
        }
    }

    /**
     * Get ItemStack of Region
     * @param region Region
     * @return ItemStack
     */
    private static ItemStack getRegion(Region region) {
        ItemBuilder builder = new ItemBuilder(region.getMaterial());
        builder.setName(region.getName());

        return builder.getItem();
    }

    /**
     * Get ItemStack of Attraction
     * @param attraction Attraction
     * @return ItemStack
     */
    private static ItemStack getAttraction(Attraction attraction) {
        ItemBuilder builder = new ItemBuilder(attraction.getStatus().getItem());
        builder.setName(attraction.getName())
                .setLore(attraction.getStatus().getColor() + attraction.getStatus().getName());

        return builder.getItem();
    }

    /**
     * Get next page MenuItem
     * @param page Integer
     * @return MenuItem
     */
    private MenuItem getNext(int page) {
        return getPage(MessageKey.MENU_NEXT.getMessage(), page + 1);
    }

    /**
     * Get previous page MenuItem
     * @param page Integer
     * @return MenuItem
     */
    private MenuItem getPrevious(int page) {
        return getPage(MessageKey.MENU_PREVIOUS.getMessage(), page - 1);
    }

    /**
     * Get next/previous page MenuItem
     * @param text String
     * @param page Integer
     * @return MenuItem
     */
    private MenuItem getPage(String text, int page) {
        ItemBuilder builder = new ItemBuilder(Material.ARROW);
        builder.setName(text);

        MenuItem item = new MenuItem(builder.getItem(), true);
        item.addActions(new PageAction(
                menu,
                page + 1
        ));
        return item;
    }

}
